-- Chapter 10 Oracle Examples
-- DROP VIEW statements are provided to allow the view to be created more than one time.
-- Examples 10.1 to 10.18 use the university database introduced in Chapter 4
-- Examples 10.34 and 10.35 use the revised university database introduced in Chapter 10.4

10.1
DROP VIEW IS_View;

CREATE VIEW IS_View AS
 SELECT * FROM Student
  WHERE StdMajor = 'IS';

10.2
DROP VIEW MS_View;

CREATE VIEW MS_View AS
 SELECT OfferNo, Offering.CourseNo, CrsUnits, OffTerm,
        OffYear, Offering.FacNo, FacFirstName,
        FacLastName, OffTime, OffDays
  FROM Faculty, Course, Offering
  WHERE FacDept = 'MS' 
    AND Faculty.FacNo = Offering.FacNo
    AND Offering.CourseNo = Course.CourseNo;

10.3a
DROP VIEW Enrollment_View;

CREATE VIEW  Enrollment_View
(OfferNo, CourseNo, Term, OffYear, Instructor, NumStudents) 
 AS
 SELECT Offering.OfferNo, CourseNo, OffTerm, OffYear,
        FacLastName, COUNT(*)
  FROM Offering, Faculty, Enrollment
  WHERE  Offering.FacNo  =  Faculty.FacNo
    AND Offering.OfferNo  =  Enrollment.OfferNo
  GROUP BY Offering.OfferNo, CourseNo, OffTerm, OffYear, 
           FacLastName;

10.3b
DROP VIEW Enrollment_View1;

CREATE VIEW  Enrollment_View AS
 SELECT Offering.OfferNo, CourseNo, OffTerm, OffYear,
        FacLastName AS Instructor, COUNT(*) AS NumStudents
  FROM Offering, Faculty, Enrollment
  WHERE  Offering.FacNo  =  Faculty.FacNo
    AND Offering.OfferNo  =  Enrollment.OfferNo
  GROUP BY Offering.OfferNo, CourseNo, OffTerm, OffYear, 
           FacLastName;

10.4
SELECT OfferNo, CourseNo, FacFirstName, FacLastName,
       OffTime, OffDays
 FROM MS_View
 WHERE OffTerm = 'SPRING' AND OffYear = 2017;

10.5
SELECT OfferNo, CourseNo, Instructor, NumStudents
 FROM Enrollment_View
 WHERE Term = 'SPRING' AND OffYear = 2017 
   AND CourseNo LIKE 'IS%';

10.6
SELECT Instructor, AVG(NumStudents) AS AvgStdCount
 FROM Enrollment_View
 GROUP BY Instructor;

10.7
SELECT OfferNo, Instructor, NumStudents, CrsUnits
 FROM Enrollment_View, Course
 WHERE Enrollment_View.CourseNo = Course.CourseNo 
   AND NumStudents < 5;

10.8
SELECT OfferNo, CourseNo, FacFirstName, FacLastName,
       OffTime, OffDays
 FROM MS_View
 WHERE OffTerm = 'SPRING' AND OffYear = 2017;

10.9
SELECT OfferNo, Course.CourseNo, FacFirstName,
       FacLastName, OffTime, OffDays
 FROM Faculty, Course, Offering
 WHERE FacDept = 'MS' 
   AND Faculty.FacNo = Offering.FacNo
   AND Offering.CourseNo = Course.CourseNo 
   AND OffTerm = 'SPRING' AND OffYear = 2017;

10.10
SELECT OfferNo, CourseNo, FacFirstName, FacLastName,
       OffTime, OffDays
 FROM Faculty, Offering
 WHERE FacDept = 'MS' 
   AND Faculty.FacNo = Offering.FacNo
   AND OffTerm = 'SPRING' AND OffYear = 2017;

10.11
CREATE VIEW Fac_View1 AS 
 SELECT FacNo, FacFirstName, FacLastName, FacRank,
        FacSalary, FacDept, FacCity, FacState, FacZipCode
  FROM Faculty
  WHERE FacDept = 'MS';

10.12
CREATE VIEW Fac_View2 AS 
 SELECT FacDept, FacRank, FacSalary 
  FROM Faculty
  WHERE FacSalary > 50000;

10.13
CREATE View Fac_View3 (FacDept, AvgSalary) AS 
 SELECT FacDept, AVG(FacSalary) 
  FROM Faculty
  WHERE FacRank = 'PROF'
  GROUP BY FacDept;

10.14
INSERT INTO Fac_View1 
 (FacNo, FacFirstName, FacLastName, FacRank, FacSalary,
  FacDept, FacCity, FacState, FacZipCode)
 VALUES ('999-99-8888', 'JOE', 'SMITH', 'PROF', 80000,
         'MS', 'SEATTLE', 'WA', '98011-011');

10.15
UPDATE Fac_View1 
 SET FacSalary = FacSalary * 1.1
 WHERE FacRank = 'ASST';

10.16
DELETE FROM Fac_View1
 WHERE FacNo = '098-76-5432';

10.17
UPDATE Fac_View1 
 SET FacDept = 'FIN'
 WHERE FacSalary > 100000;

10.18
CREATE VIEW Fac_View1_Revised AS 
 SELECT FacNo, FacFirstName, FacLastName, FacRank,
        FacSalary, FacDept, FacCity, FacState, FacZipCode
  FROM Faculty
  WHERE FacDept = 'MS'
 WITH CHECK OPTION;

UPDATE Fac_View1_Revised 
 SET FacDept = 'FIN'
 WHERE FacSalary > 100000;

-- Examples 10.38 and 10.39 use the revised university database introduced in Section 10.4
10.38
SELECT Faculty.FacNo, Faculty.FacFirstName, FacLastName,
       Faculty.FacDept, Offering.OfferNo,
       Offering.CourseNo, Offering.OffTerm,
       Offering.OffYear, Offering.OffLocation,
       Offering.OffTime, Offering.OffDays
 FROM Faculty, Offering
 WHERE Faculty.FacNo = Offering.FacNo
   AND ( ( Offering.OffTerm = 'FALL' 
   AND Offering.OffYear = 2016 ) 
    OR ( Offering.OffTerm = 'WINTER' 
   AND Offering.OffYear = 2017 ) 
    OR ( Offering.OffTerm = 'SPRING' 
   AND Offering.OffYear = 2017 ) );

10.39 (Access)
SELECT Offering.OfferNo, FacFirstName, FacLastName,
       FacDept, OffTerm, CrsUnits, OffLimit,
       Count(Enrollment.RegNo) AS NumStds, 
       NumStds/Offlimit AS PercentFull, 
       (NumStds/Offlimit) < 0.25 AS LowEnrollment
 FROM Faculty, Offering, Course, Enrollment
 WHERE Faculty.FacNo = Offering.FacNo 
   AND Course.CourseNo = Offering.CourseNo
   AND Offering.OfferNo = Enrollment.OfferNo
   AND ( ( Offering.OffTerm = 'FALL' 
   AND Offering.OffYear = 2016 ) 
    OR ( Offering.OffTerm = 'WINTER' 
          AND Offering.OffYear = 2017 ) 
    OR ( Offering.OffTerm = 'SPRING' 
   AND Offering.OffYear = 2017 ) )
 GROUP BY Offering.OfferNo, FacFirstName, FacLastName,
          FacDept, OffTerm, CrsUnits, OffLimit;

10.39 (Oracle)
-- Uses a CASE expression 
SELECT Offering.OfferNo, FacFirstName, FacLastName,
       FacDept, OffTerm, CrsUnits, OffLimit,
       Count(Enrollment.RegNo) AS NumStds, 
       Count(Enrollment.RegNo)/Offlimit AS PercentFull, 
       CASE WHEN Count(Enrollment.RegNo)/Offlimit < 0.25  THEN 'TRUE' ELSE 'FALSE' END AS LowEnrollment
 FROM Faculty, Offering, Course, Enrollment
 WHERE Faculty.FacNo = Offering.FacNo 
   AND Course.CourseNo = Offering.CourseNo
   AND Offering.OfferNo = Enrollment.OfferNo
   AND ( ( Offering.OffTerm = 'FALL' 
   AND Offering.OffYear = 2016 ) 
    OR ( Offering.OffTerm = 'WINTER' 
          AND Offering.OffYear = 2017 ) 
    OR ( Offering.OffTerm = 'SPRING' 
   AND Offering.OffYear = 2017 ) )
 GROUP BY Offering.OfferNo, FacFirstName, FacLastName,
          FacDept, OffTerm, CrsUnits, OffLimit;


10.B1
CREATE VIEW Course_Offering_View AS
  SELECT Course.CourseNo, CrsDesc, CrsUnits,
         Offering.OfferNo, OffTerm, OffYear,
         OffLocation, OffTime, FacNo, OffDays
	FROM Course INNER JOIN Offering 
        ON Course.CourseNo = Offering.CourseNo;
